@@ -136,7 +136,7 @@ atomicElementListClass <- function(x) {
   ans
 }
 
-### FIXME: these seem very similar to asList().
+### FIXME: these seem very similar to .asList().
 
 SimpleAtomicList <- function(listData) {
   classOrder <-
@@ -257,6 +257,7 @@ setAtomicListCoercions <- function(type) {
   setAs("ANY", CompressedClass, CoercerToAtomicList(type, compress = TRUE))
   setAs("ANY", SimpleClass, CoercerToAtomicList(type, compress = FALSE))
   setAs("ANY", Class, CoercerToAtomicList(type, compress = TRUE))
+  setAs("SimpleList", Class, CoercerToAtomicList(type, compress = FALSE))
 }
 
 setAtomicListCoercions("logical")
@@ -673,67 +674,6 @@ setAtomicListMethod <- function(f,
 ### General methods
 ###
 
-setAtomicListMethod("is.na", outputBaseClass = "LogicalList",
-                    applyToUnlist = TRUE)
-
-### S3/S4 combo for duplicated.AtomicList
-duplicated.AtomicList <- function(x, incomparables=FALSE,
-                                     fromLast=FALSE, ...)
-{
-    if (is(x, "CompressedList"))
-        ans_class <- "CompressedLogicalList"
-    else
-        ans_class <- "SimpleLogicalList"
-    ans_listData <- lapply(x, duplicated, incomparables=incomparables,
-                                          fromLast=fromLast, ...)
-    newList(ans_class, ans_listData)
-}
-setMethod("duplicated", "AtomicList", duplicated.AtomicList)
-
-### S3/S4 combo for duplicated.CompressedAtomicList
-.duplicated.CompressedAtomicList <- function(x, incomparables=FALSE,
-                                                fromLast=FALSE)
-{
-    if (!identical(incomparables, FALSE))
-        stop("\"duplicated\" method for CompressedAtomicList objects ",
-             "does not support the 'incomparables' argument")
-    x_unlistData <- x@unlistData
-    sm <- match(x_unlistData, x_unlistData)  # doesn't work on an Rle
-    x_group <- rep.int(seq_along(x), elementLengths(x))
-    ans_unlistData <- duplicatedIntegerPairs(x_group, sm, fromLast=fromLast)
-    relist(ans_unlistData, x)
-}
-duplicated.CompressedAtomicList <- function(x, incomparables=FALSE,
-                                               fromLast=FALSE,
-                                               ...)
-    .duplicated.CompressedAtomicList(x, incomparables=incomparables,
-                                        fromLast=fromLast,
-                                        ...)
-setMethod("duplicated", "CompressedAtomicList",
-          duplicated.CompressedAtomicList)
-
-### S3/S4 combo for unique.CompressedAtomicList
-.unique.CompressedAtomicList <- function(x, incomparables=FALSE,
-                                            fromLast=FALSE)
-{
-    if (!identical(incomparables, FALSE))
-        stop("\"unique\" method for CompressedAtomicList objects ",
-             "does not support the 'incomparables' argument")
-    is_dup <- duplicated(x, incomparables=incomparables, fromLast=fromLast)
-    x_unlistData <- x@unlistData
-    keep_idx <- which(!is_dup@unlistData)
-    ans_unlistData <- x_unlistData[keep_idx]
-    x_group <- rep.int(seq_along(x), elementLengths(x))
-    ans_group <- x_group[keep_idx]
-    ans_eltlens <- tabulate(ans_group, nbins=length(x))
-    ans_partitioning <- PartitioningByEnd(cumsum(ans_eltlens),
-                                          names=names(x))
-    relist(ans_unlistData, ans_partitioning)
-}
-unique.CompressedAtomicList <- function(x, incomparables=FALSE, ...)
-    .unique.CompressedAtomicList(x, incomparables=incomparables, ...)
-setMethod("unique", "CompressedAtomicList", unique.CompressedAtomicList)
-
 ### S3/S4 combo for unique.CompressedRleList
 unique.CompressedRleList <- function(x, incomparables=FALSE, ...)
 {
@@ -757,16 +697,6 @@ unique.SimpleRleList <- function(x, incomparables=FALSE, ...)
 }
 setMethod("unique", "SimpleRleList", unique.SimpleRleList)
 
-setAtomicListMethod("match", outputBaseClass = "IntegerList",
-                    remainingSignature = "atomic")
-setAtomicListMethod("match", outputBaseClass = "IntegerList",
-                    remainingSignature = "AtomicList", mapply = TRUE)
-
-setAtomicListMethod("%in%", outputBaseClass = "LogicalList",
-                    remainingSignature = "atomic")
-setAtomicListMethod("%in%", outputBaseClass = "LogicalList",
-                    remainingSignature = "AtomicList", mapply = TRUE)
-
 setMethod("table", "SimpleAtomicList",
           function(...)
           {
@@ -806,20 +736,11 @@ setMethod("table", "CompressedAtomicList",
               ans
           })
 
-setAtomicListMethod("order", outputBaseClass = "IntegerList")
-
-### S3/S4 combo for sort.RleList
-sort.RleList <- function(x, decreasing=FALSE, ...)
-    endoapply(x, sort, decreasing=decreasing, ...)
-setMethod("sort", "RleList", sort.RleList)
-
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Logical methods
 ###
 
-setAtomicListMethod("!", inputBaseClass = "LogicalList", 
-                    outputBaseClass = "LogicalList", applyToUnlist = TRUE)
 setAtomicListMethod("which", inputBaseClass = "LogicalList",
                     outputBaseClass = "IntegerList",
                     rleListOutputBaseClass = "IntegerList")
@@ -977,8 +898,20 @@ setMethod("runq", "RleList",
 ### Character
 ###
 
-setAtomicListMethod("nchar", inputBaseClass = "CharacterList",
-                    outputBaseClass = "IntegerList", applyToUnlist = TRUE)
+nchar_CompressedList <- function(x, type="chars", allowNA=FALSE)
+{
+        unlisted_x <- unlist(x, use.names=FALSE)
+        unlisted_ans <- nchar(unlisted_x, type=type, allowNA=allowNA)
+        relist(unlisted_ans, x)
+}
+
+setMethod("nchar", "CompressedCharacterList", nchar_CompressedList)
+setMethod("nchar", "SimpleCharacterList", nchar_CompressedList)
+
+setMethod("nchar", "CompressedRleList", nchar_CompressedList)
+setMethod("nchar", "SimpleRleList", nchar_CompressedList)
+
+
 ## need vectorized start, end
 ##setAtomicListMethod("substr")
 ##setAtomicListMethod("substring")
@@ -997,6 +930,10 @@ setAtomicListMethod("gsub", inputBaseClass = "CharacterList",
                     applyToUnlist = TRUE)
 ### TODO: grep, grepl
 
+setMethod("unstrsplit", "CharacterList",
+    function(x, sep="") unstrsplit(as.list(x), sep=sep)
+)
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Rle methods
 ###
@@ -1025,6 +962,27 @@ setMethod("runValue", "CompressedRleList",
     }
 )
 
+setReplaceMethod("runValue", "CompressedRleList",
+                 function(x, value) {
+                   if (!identical(elementLengths(ranges(x)),
+                                  elementLengths(value)))
+                     stop("elementLengths() of 'x' and 'value' must match")
+                   runValue(x@unlistData) <- unlist(value, use.names=FALSE)
+                   x
+                 })
+
+setReplaceMethod("runValue", "SimpleRleList",
+                 function(x, value) {
+                   if (!identical(elementLengths(ranges(x)),
+                                  elementLengths(value)))
+                     stop("elementLengths() of 'x' and 'value' must match")
+                   x@listData <- mapply(function(rle, v) {
+                     runValue(rle) <- v
+                     rle
+                   }, x, value, SIMPLIFY=FALSE)
+                   x
+                 })
+
 setMethod("runLength", "RleList", function(x) {
   seqapply(x, runLength)
 })
