@@ -230,10 +230,7 @@ setReplaceMethod("[[", "DataFrame",
                      else
                        value <- rep(value, length.out = nrx)
                    }
-                   x <- callNextMethod(x, i, value=value)
-                   ## ensure unique, valid names
-                   names(x) <- make.names(names(x), unique=TRUE)
-                   x
+                   callNextMethod(x, i, value=value)
                  })
 
 setMethod("extractROWS", "DataFrame",
@@ -471,6 +468,26 @@ setReplaceMethod("[", "DataFrame",
                    x
                  })
 
+hasNonDefaultMethod <- function(f, signature) {
+  any(selectMethod(f, signature)@defined != "ANY")
+}
+
+hasS3Method <- function(f, signature) {
+  !is.null(getS3method(f, signature, optional=TRUE))
+}
+
+droplevels.DataFrame <- function(x, except=NULL) {
+  canDropLevels <- function(xi) {
+    hasNonDefaultMethod(droplevels, class(xi)) ||
+      hasS3Method("droplevels", class(xi))
+  }
+  drop.levels <- vapply(x, canDropLevels, NA)
+  if (!is.null(except)) 
+    drop.levels[except] <- FALSE
+  x@listData[drop.levels] <- lapply(x@listData[drop.levels], droplevels)
+  x
+}
+setMethod("droplevels", "DataFrame", droplevels.DataFrame)
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Coercion.
@@ -527,9 +544,13 @@ setAs("data.frame", "DataFrame",
         rn <- attributes(from)[["row.names"]]
         if (is.integer(rn))
           rn <- NULL
-        rownames(from) <- NULL
-        new2("DataFrame", listData = as.list(from),
-             nrows = nrow(from), rownames = rn, check=FALSE)
+        nr <- nrow(from)
+### FIXME: this should be:
+        ## from <- as.list(from)
+### But unclass() causes deep copy
+        attr(from, "row.names") <- NULL
+        class(from) <- NULL
+        new2("DataFrame", listData=from, nrows=nr, rownames=rn, check=FALSE)
       })
 
 # matrices and tables just go through data.frame
@@ -549,6 +570,7 @@ setAs("table", "DataFrame",
         do.call(DataFrame, c(df[1], lapply(df[factors], Rle), df["Freq"]))
       })
 
+setOldClass(c("xtabs", "table"))
 setAs("xtabs", "DataFrame",
       function(from) {
         class(from) <- "table"
