@@ -151,6 +151,12 @@ newList <- function(Class, listData, ..., mcols)
     reconcileMetadatacols(ans)
 }
 
+List <- function(...) {
+  list <- list(...)
+  if (length(list) == 1 && is.list(list[[1L]])) 
+    list <- list[[1L]]
+  as(list, "List")
+}
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### The "show" method.
@@ -168,31 +174,167 @@ setMethod("show", "List",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### List-like API: Element extraction.
+### Subsetting.
 ###
 
-setMethod("$", "List", function(x, name) x[[name, exact=FALSE]])
+### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+### Returns TRUE iff 'i' contains non-NA positive values that are compatible
+### with the shape of 'x'.
+.is_valid_NL_subscript <- function(i, x)
+{
+    unlisted_i <- unlist(i, use.names=FALSE)
+    if (!is.integer(unlisted_i))
+        unlisted_i <- as.integer(unlisted_i)
+    if (anyMissingOrOutside(unlisted_i, lower=1L))
+        return(FALSE)
+    x_eltlens <- elementLengths(x)
+    i_eltlens <- elementLengths(i)
+    if (any(unlisted_i > rep.int(x_eltlens, i_eltlens)))
+        return(FALSE)
+    return(TRUE)
+}
 
-### Assumes 'i' to be either a LogicalList or a logical-RleList of the same
-### length as 'x'. Truncate or recycle each list element of 'i' to the length
-### of the corresponding element in 'x'.
-.normalizeLogicalListSubscript <- function(i, x)
+### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+### Returns the name of one of the 3 supported fast paths ("LL", "NL", "RL")
+### or NA if no fast path can be used.
+.select_fast_path <- function(i, x)
+{
+    ## LEPType (List Element Pseudo-Type): same as "elementType" except for
+    ## RleList objects.
+    if (is(i, "RleList")) {
+        i_runvals <- runValue(i)
+        i_LEPType <- elementType(i_runvals)
+    } else {
+        i_LEPType <- elementType(i)
+    }
+    if (extends(i_LEPType, "logical")) {
+        ## 'i' is a List of logical vectors or logical-Rle objects.
+        ## We select the "LL" fast path ("Logical List").
+        return("LL")
+    }
+    if (extends(i_LEPType, "numeric")) {
+        ## 'i' is a List of numeric vectors or numeric-Rle objects.
+        if (is(i, "RleList")) {
+            i2 <- i_runvals
+        } else {
+            i2 <- i
+        }
+        if (.is_valid_NL_subscript(i2, x)) {
+            ## We select the "NL" fast path ("Number List").
+            return("NL")
+        }
+    }
+    if (extends(i_LEPType, "Ranges")) {
+        ## 'i' is a List of Ranges objects.
+        ## We select the "RL" fast path ("Ranges List").
+        return("RL")
+    }
+    return(NA_character_)
+}
+
+### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+### Truncate or recycle each list element of 'i' to the length of the
+### corresponding element in 'x'.
+.adjust_elt_lengths <- function(i, x)
 {
     x_eltlens <- unname(elementLengths(x))
     i_eltlens <- unname(elementLengths(i))
     idx <- which(x_eltlens != i_eltlens)
     ## FIXME: This is rough and doesn't follow exactly the truncate-or-recycle
     ## semantic of normalizeSingleBracketSubscript() on a logical vector or
-    ## logical Rle.
+    ## logical-Rle object.
     for (k in idx)
         i[[k]] <- rep(i[[k]], length.out=x_eltlens[k])
     return(i)
 }
 
+### Assumes 'x' and 'i' are parallel List objects (i.e. same length),
+### and 'i' is a List of logical vectors or logical-Rle objects.
+.unlist_LL_subscript <- function(i, x)
+{
+    i <- .adjust_elt_lengths(i, x)
+    unlist(i, use.names=FALSE)
+}
+
+### Assumes 'x' and 'i' are parallel List objects (i.e. same length),
+### and 'i' is a List of numeric vectors or numeric-Rle objects.
+.unlist_NL_subscript <- function(i, x)
+{
+    offsets <- c(0L, end(PartitioningByEnd(x))[-length(x)])
+    i <- i + offsets
+    unlist(i, use.names=FALSE)
+}
+
+### Assumes 'x' and 'i' are parallel List objects (i.e. same length),
+### and 'i' is a List of Ranges objects.
+.unlist_RL_subscript <- function(i, x)
+{
+    unlisted_i <- unlist(i, use.names=FALSE)
+    offsets <- c(0L, end(PartitioningByEnd(x))[-length(x)])
+    shift(unlisted_i, shift=rep.int(offsets, elementLengths(i)))
+}
+
+### Fast subset by List of logical vectors or logical-Rle objects.
+### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+.fast_subset_List_by_LL <- function(x, i)
+{
+    ## Unlist 'x' and 'i'.
+    unlisted_x <- unlist(x, use.names=FALSE)
+    unlisted_i <- .unlist_LL_subscript(i, x)
+
+    ## Subset.
+    unlisted_ans <- extractROWS(unlisted_x, unlisted_i)
+
+    ## Relist.
+    group <- rep.int(seq_along(x), elementLengths(x))
+    group <- extractROWS(group, unlisted_i)
+    ans_skeleton <- PartitioningByEnd(group, NG=length(x), names=names(x))
+    ans <- as(relist(unlisted_ans, ans_skeleton), class(x))
+    metadata(ans) <- metadata(x)
+    ans
+}
+
+### Fast subset by List of numeric vectors or numeric-Rle objects.
+### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+.fast_subset_List_by_NL <- function(x, i)
+{
+    ## Unlist 'x' and 'i'.
+    unlisted_x <- unlist(x, use.names=FALSE)
+    unlisted_i <- .unlist_NL_subscript(i, x)
+
+    ## Subset.
+    unlisted_ans <- extractROWS(unlisted_x, unlisted_i)
+
+    ## Relist.
+    ans_breakpoints <- cumsum(unname(elementLengths(i)))
+    ans_skeleton <- PartitioningByEnd(ans_breakpoints, names=names(x))
+    ans <- as(relist(unlisted_ans, ans_skeleton), class(x))
+    metadata(ans) <- metadata(x)
+    ans
+}
+
+### Fast subset by List of Ranges objects.
+### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+.fast_subset_List_by_RL <- function(x, i)
+{
+    ## Unlist 'x' and 'i'.
+    unlisted_x <- unlist(x, use.names=FALSE)
+    unlisted_i <- .unlist_RL_subscript(i, x)
+
+    ## Subset.
+    unlisted_ans <- extractROWS(unlisted_x, unlisted_i)
+
+    ## Relist.
+    ans_breakpoints <- cumsum(unlist(sum(width(i)), use.names=FALSE))
+    ans_skeleton <- PartitioningByEnd(ans_breakpoints, names=names(x))
+    ans <- as(relist(unlisted_ans, ans_skeleton), class(x))
+    metadata(ans) <- metadata(x)
+    ans
+}
+
 ### Subset a List object by a list-like subscript.
-subsetListByList <- function(x, i)
+subset_List_by_List <- function(x, i)
 {
-    x_class <- class(x)
     li <- length(i)
     if (is.null(names(i))) {
         lx <- length(x)
@@ -206,72 +348,83 @@ subsetListByList <- function(x, i)
             stop("cannot subscript an unnamed list-like object ",
                  "by a named list-like object")
         if (!identical(names(i), names(x))) {
-            j <- match(names(i), names(x))
-            if (anyMissing(j))
+            i2x <- match(names(i), names(x))
+            if (anyMissing(i2x))
                 stop("list-like subscript has names not in ",
                      "list-like object to subset")
-            x <- x[j]
+            x <- x[i2x]
         }
     }
     ## From here, 'x' and 'i' are guaranteed to have the same length.
     if (li == 0L)
         return(x)
     if (!is(x, "SimpleList")) {
-        if (!is(i, "List"))
-            i <- as(i, "List")
-        ## List element pseudo-type: same as "elementType" except for RleList
-        ## objects.
-        if (is(i, "RleList"))
-            leptype <- elementType(runValue(i))
-        else
-            leptype <- elementType(i)
-        x_names <- names(x)
-        if (extends(leptype, "logical")) {
-            unlisted_x <- unlist(x, use.names=FALSE)
-            i <- .normalizeLogicalListSubscript(i, x)
-            unlisted_i <- unlist(i, use.names=FALSE)
-            unlisted_ans <- extractROWS(unlisted_x, unlisted_i)
-            group <- rep.int(seq_along(x), elementLengths(x))
-            group <- extractROWS(group, unlisted_i)
-            ans_skeleton <- PartitioningByEnd(group, NG=length(x),
-                                              names=x_names)
-            ans <- as(relist(unlisted_ans, ans_skeleton), x_class)
-            metadata(ans) <- metadata(x)
-            return(ans)
+        ## We'll try to take a fast path.
+        if (is(i, "List")) {
+            fast_path <- .select_fast_path(i, x)
+        } else {
+            i2 <- as(i, "List")
+            i2_elttype <- elementType(i2)
+            if (length(i2) == li && all(sapply(i, is, i2_elttype))) {
+                fast_path <- .select_fast_path(i2, x)
+                if (!is.na(fast_path))
+                    i <- i2
+            } else {
+                fast_path <- NA_character_
+            }
         }
-        if (extends(leptype, "numeric")) {
-            unlisted_x <- unlist(x, use.names=FALSE)
-            offsets <- c(0L, end(PartitioningByEnd(x))[-length(x)])
-            i <- i + offsets
-            unlisted_i <- unlist(i, use.names=FALSE)
-            unlisted_ans <- extractROWS(unlisted_x, unlisted_i)
-            ans_breakpoints <- cumsum(unname(elementLengths(i)))
-            ans_skeleton <- PartitioningByEnd(ans_breakpoints, names=x_names)
-            ans <- as(relist(unlisted_ans, ans_skeleton), x_class)
-            metadata(ans) <- metadata(x)
-            return(ans)
-        }
-        if (extends(leptype, "Ranges")) {
-            unlisted_x <- unlist(x, use.names=FALSE)
-            offsets <- c(0L, end(PartitioningByEnd(x))[-length(x)])
-            unlisted_i <- unlist(i, use.names=FALSE)
-            unlisted_i <- shift(unlisted_i,
-                                shift=rep.int(offsets, elementLengths(i)))
-            unlisted_ans <- extractROWS(unlisted_x, unlisted_i)
-            ans_breakpoints <- cumsum(unlist(sum(width(i)), use.names=FALSE))
-            ans_skeleton <- PartitioningByEnd(ans_breakpoints, names=x_names)
-            ans <- as(relist(unlisted_ans, ans_skeleton), x_class)
-            metadata(ans) <- metadata(x)
-            return(ans)
+        if (!is.na(fast_path)) {
+            fast_path_FUN <- switch(fast_path,
+                                    LL=.fast_subset_List_by_LL,
+                                    NL=.fast_subset_List_by_NL,
+                                    RL=.fast_subset_List_by_RL)
+            return(fast_path_FUN(x, i))  # fast path
         }
     }
-    ## NOT efficient because it loops over the elements of 'x'.
+    ## Slow path (loops over the list elements of 'x').
     for (k in seq_len(li))
         x[[k]] <- extractROWS(x[[k]], i[[k]])
     return(x)
 }
 
-subsetListByList_replace <- function(x, i, value, byrow=FALSE)
+.adjust_value_length <- function(value, i_len)
+{
+    value_len <- length(value)
+    if (value_len == i_len)
+        return(value)
+    if (i_len %% value_len != 0L)
+        warning("number of values supplied is not a sub-multiple ",
+                "of the number of values to be replaced")
+    rep(value, length.out=i_len)
+}
+
+### Assumes 'x' and 'i' are parallel List objects (i.e. same length).
+.fast_lsubset_List_by_List <- function(x, i, value)
+{
+    ## Unlist 'x', 'i', and 'value'.
+    unlisted_x <- unlist(x, use.names=FALSE)
+    fast_path <- .select_fast_path(i, x)
+    unlist_subscript_FUN <- switch(fast_path,
+                                   LL=.unlist_LL_subscript,
+                                   NL=.unlist_NL_subscript,
+                                   RL=.unlist_RL_subscript)
+    unlisted_i <- unlist_subscript_FUN(i, x)
+    if (length(value) != 1L) {
+        value <- .adjust_value_length(value, length(i))
+        value <- .adjust_elt_lengths(value, i)
+    }
+    unlisted_value <- unlist(value, use.names=FALSE)
+
+    ## Subset.
+    unlisted_ans <- replaceROWS(unlisted_x, unlisted_i, unlisted_value)
+
+    ## Relist.
+    ans <- as(relist(unlisted_ans, x), class(x))
+    metadata(ans) <- metadata(x)
+    ans
+}
+
+lsubset_List_by_List <- function(x, i, value)
 {
     lx <- length(x)
     li <- length(i)
@@ -284,40 +437,44 @@ subsetListByList_replace <- function(x, i, value, byrow=FALSE)
     if (lv == 0L)
         stop("replacement has length zero")
     value <- normalizeSingleBracketReplacementValue(value, x)
-    if (li != lv) {
-        if (li %% lv != 0L)
-            warning("number of values supplied is not a sub-multiple ",
-                    "of the number of values to be replaced")
-        value <- rep(value, length.out = li)
-    }
     if (is.null(names(i))) {
-        if (li > lx)
-            stop("list-like subscript is longer than ",
-                 "list-like object to subset")
-        for (ii in seq_len(li)) {
-            xx <- x[[ii]]
-            if (byrow)
-                xx[i[[ii]], ] <- value[[ii]]
-            else
-                xx[i[[ii]]] <- value[[ii]]
-            x[[ii]] <- xx
+        if (li != lx)
+            stop("when list-like subscript is unnamed, it must have the ",
+                 "length of list-like object to subset")
+        if (!is(x, "SimpleList")) {
+            ## We'll try to take a fast path.
+            if (is(i, "List")) {
+                fast_path <- .select_fast_path(i, x)
+            } else {
+                i2 <- as(i, "List")
+                i2_elttype <- elementType(i2)
+                if (length(i2) == li && all(sapply(i, is, i2_elttype))) {
+                    fast_path <- .select_fast_path(i2, x)
+                    if (!is.na(fast_path))
+                        i <- i2
+                } else {
+                    fast_path <- NA_character_
+                }
+            }
+            if (!is.na(fast_path))
+                return(.fast_lsubset_List_by_List(x, i, value))  # fast path
         }
-        return(x)
-    }
-    if (is.null(names(x)))
-        stop("cannot subscript an unnamed list-like object ",
-             "by a named list-like object")
-    j <- match(names(i), names(x))
-    if (anyMissing(j))
-        stop("list-like subscript has names not in list-like object to subset")
-    for (ii in seq_len(li)) {
-        xx <- x[[j[ii]]]
-        if (byrow)
-            xx[i[[ii]], ] <- value[[ii]]
-        else
-            xx[i[[ii]]] <- value[[ii]]
-        x[[j[ii]]] <- xx
+        i2x <- seq_len(li)
+    } else {
+        if (is.null(names(x)))
+            stop("cannot subset an unnamed list-like object ",
+                 "by a named list-like subscript")
+        i2x <- match(names(i), names(x))
+        if (anyMissing(i2x))
+            stop("list-like subscript has names not in ",
+                 "list-like object to subset")
+        if (anyDuplicated(i2x))
+            stop("list-like subscript has duplicated names")
     }
+    value <- .adjust_value_length(value, li)
+    ## Slow path (loops over the list elements of 'x').
+    for (k in seq_len(li))
+        x[[i2x[k]]] <- replaceROWS(x[[i2x[k]]], i[[k]], value[[k]])
     return(x)
 }
 
@@ -327,7 +484,7 @@ setMethod("[", "List",
         if (!missing(j) || length(list(...)) > 0L)
             stop("invalid subsetting")
         if (!missing(i) && (is.list(i) || (is(i, "List") && !is(i, "Ranges"))))
-            return(subsetListByList(x, i))
+            return(subset_List_by_List(x, i))
         callNextMethod(x, i)
     }
 )
@@ -338,7 +495,7 @@ setReplaceMethod("[", "List",
         if (!missing(j) || length(list(...)) > 0L)
             stop("invalid subsetting")
         if (!missing(i) && (is.list(i) || (is(i, "List") && !is(i, "Ranges"))))
-                return(subsetListByList_replace(x, i, value))
+                return(lsubset_List_by_List(x, i, value))
         callNextMethod(x, i, value=value)
     }
 )
@@ -356,6 +513,8 @@ setMethod("[[", "List",
     }
 )
 
+setMethod("$", "List", function(x, name) x[[name, exact=FALSE]])
+
 setReplaceMethod("[[", "List",
                  function(x, i, j, ..., value)
                  {
@@ -374,6 +533,60 @@ setReplaceMethod("$", "List",
                    x
                  })
 
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Simple helper functions for some common subsetting operations.
+###
+
+### phead() and ptail(): "parallel" versions of head() and tail() for List
+### objects. They're just fast equivalents of 'mapply(head, x, n)' and
+### 'mapply(tail, x, n)', respectively.
+.normarg_n <- function(n, x_eltlens)
+{
+    if (!is.numeric(n))
+        stop("'n' must be an integer vector")
+    if (!is.integer(n))
+        n <- as.integer(n)
+    if (any(is.na(n)))
+        stop("'n' cannot contain NAs")
+    n <- pmin(x_eltlens, n)
+    neg_idx <- which(n < 0L)
+    if (length(neg_idx) != 0L)
+        n[neg_idx] <- pmax(n[neg_idx] + x_eltlens[neg_idx], 0L)
+    n
+}
+
+phead <- function(x, n=6L)
+{
+    x_eltlens <- unname(elementLengths(x))
+    n <- .normarg_n(n, x_eltlens)
+    unlisted_i <- IRanges(start=rep.int(1L, length(n)), width=n)
+    i <- relist(unlisted_i, PartitioningByEnd(seq_along(x)))
+    x[i]
+}
+
+ptail <- function(x, n=6L)
+{
+    x_eltlens <- unname(elementLengths(x))
+    n <- .normarg_n(n, x_eltlens)
+    unlisted_i <- IRanges(end=x_eltlens, width=n)
+    i <- relist(unlisted_i, PartitioningByEnd(seq_along(x)))
+    x[i]
+}
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### compareRecursively()
+###
+### NOT exported!
+### See R/List-comparison.R for the details.
+###
+
+setGeneric("compareRecursively",
+    function(x) standardGeneric("compareRecursively")
+)
+
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Looping methods.
 ###
@@ -391,8 +604,8 @@ setMethod("lapply", "List",
 environment(.sapplyDefault) <- topenv()
 setMethod("sapply", "List", .sapplyDefault)
 
-.mapply_List <- function(FUN, ..., MoreArgs = NULL, SIMPLIFY = TRUE,
-                         USE.NAMES = TRUE)
+mapply_List <- function(FUN, ..., MoreArgs = NULL, SIMPLIFY = TRUE,
+                        USE.NAMES = TRUE)
 {
     seqs <- list(...)
     isListOrVector <- function(x) is.vector(x) | is(x, "List")
@@ -416,8 +629,6 @@ setMethod("sapply", "List", .sapplyDefault)
            USE.NAMES = USE.NAMES)
 }
 
-setMethod("mapply", "List", .mapply_List)
-
 setMethod("endoapply", "List",
           function(X, FUN, ...) {
               elementTypeX <- elementType(X)
@@ -435,6 +646,7 @@ setMethod("mendoapply", "List",
           function(FUN, ..., MoreArgs = NULL) {
               X <- list(...)[[1L]]
               elementTypeX <- elementType(X)
+              FUN <- match.fun(FUN)
               listData <-
                 mapply(FUN = FUN, ..., MoreArgs = MoreArgs, SIMPLIFY = FALSE)
               for (i in seq_len(length(listData))) {
@@ -445,87 +657,6 @@ setMethod("mendoapply", "List",
               X
           })
 
-asList <- function(x, ...) {
-  if (is(x, "List"))
-    return(x)
-  if (!is.list(x))
-    stop("'x' must be a 'list'")
-  cl <- lapply(x, class)
-  clnames <- unique(unlist(cl, use.names=FALSE))
-  cons <- SimpleList
-  if (length(clnames) == 1L) {
-    cl <- cl[[1]]
-    pkg <- packageSlot(cl)
-  } else if (length(clnames)) {
-    contains <- lapply(cl, function(x) getClass(x, TRUE)@contains)
-    clnames <- c(clnames,
-                 unlist(lapply(contains, names), use.names=FALSE))
-    contab <- table(factor(clnames, unique(clnames)))
-    cl <- names(contab)[contab == length(x)]
-    if (length(cl))
-      pkg <- sapply(do.call(c, unname(contains))[cl], packageSlot)
-  }
-  if (length(cl)) {
-    constructorName <- function(x) {
-      substring(x, 1, 1) <- toupper(substring(x, 1, 1))
-      paste(x, "List", sep = "")
-    }
-    if (is.null(pkg))
-      ns <- topenv()
-    else ns <- getNamespace(pkg[1])
-    consym <- constructorName(cl[1])
-    if (exists(consym, ns))
-      cons <- get(consym, ns)
-    else {
-      if (length(cl) == 1L) {
-        contains <- getClass(cl, TRUE)@contains
-        cl <- names(contains)
-        pkg <- sapply(contains, packageSlot)
-      } else {
-        cl <- tail(cl, -1)
-        pkg <- tail(pkg, -1)
-      }
-      if (length(cl)) {
-        if (!length(pkg))
-          ns <- list(topenv())
-        connms <- constructorName(cl)
-        ns <- lapply(pkg, getNamespace)
-        coni <- head(which(mapply(exists, connms, ns)), 1)
-        if (length(coni))
-          cons <- get(connms[coni], ns[[coni]])
-      }
-    }
-  }
-  cons(x, ...)
-}
-
-## FIXME: these functions should probably be renamed to c[apply], i.e.,
-## clapply, cmapply, ctapply, csplit, cby.
-
-seqapply <- function(X, FUN, ...) {
-  asList(lapply(X, FUN, ...))
-}
-
-mseqapply <- function(FUN, ..., MoreArgs = NULL, USE.NAMES = TRUE) {
-  asList(.mapply_List(FUN, ..., MoreArgs = MoreArgs, SIMPLIFY = FALSE,
-                        USE.NAMES = USE.NAMES))
-}
-
-tseqapply <- function(X, INDEX, FUN = NULL, ...) {
-  asList(tapply(X, INDEX, FUN, ..., simplify = FALSE))
-}
-
-seqsplit <- function(x, f, drop=FALSE) {
-  ans_class <- try(splitAsListReturnedClass(x), silent=TRUE)
-  if (inherits(ans_class, "try-error"))
-    return(asList(split(x, f, drop)))
-  splitAsList(x, f, drop=drop)
-}
-
-seqby <- function(data, INDICES, FUN, ...) {
-  asList(by(data, INDICES, FUN, ..., simplify = FALSE))
-}
-
 setGeneric("revElements", signature="x",
     function(x, i) standardGeneric("revElements")
 )
@@ -563,10 +694,8 @@ setAs("List", "list", function(from) as.list(from))
 as.list.List <- function(x, ...) .as.list.List(x, ...)
 setMethod("as.list", "List", as.list.List)
 
-setGeneric("as.env", function(x, ...) standardGeneric("as.env"))
-
 setMethod("as.env", "List",
-          function(x, enclos = parent.frame(2)) {
+          function(x, enclos = parent.frame(2), tform = identity) {
               nms <- names(x)
               if (is.null(nms))
                   stop("cannot convert to environment when names are NULL")
@@ -574,7 +703,7 @@ setMethod("as.env", "List",
               lapply(nms,
                      function(col) {
                          colFun <- function() {
-                             val <- x[[col]]
+                             val <- tform(x[[col]])
                              rm(list=col, envir=env)
                              assign(col, val, env)
                              val
@@ -585,28 +714,56 @@ setMethod("as.env", "List",
           })
 
 listClassName <- function(impl, element.type) {
-  if (is.null(impl))
-    impl <- ""
+  listClass <- paste0(if (is.null(impl)) "Simple" else impl, "List")
   if (!is.null(element.type)) {
     cl <- c(element.type, names(getClass(element.type)@contains))
     cl <- capitalize(cl)
-  } else {
-    cl <- ""
+    listClass <- c(paste0(cl, "List"), paste0(cl, "Set"),
+                   paste0(impl, cl, "List"), listClass)
   }
-  listClass <- c(paste0(impl, cl, "List"), paste0(cl, "List"))
   clExists <- which(sapply(listClass, isClass) &
                     sapply(listClass, extends, paste0(impl, "List")))
-  if (length(clExists) == 0L) {
-    stop("Could not find a '", impl,
-         "List' subclass for values of type '", cl, "'")
+  listClass[[clExists[[1L]]]]
+}
+
+selectListClassName <- function(x) {
+  cn <- listClassName("Compressed", x)
+  if (cn == "CompressedList")
+    cn <- listClassName(NULL, x)
+  cn
+}
+
+listElementType <- function(x) {
+  cl <- lapply(x, class)
+  clnames <- unique(unlist(cl, use.names=FALSE))
+  if (length(clnames == 1L)) {
+    clnames
+  } else {
+    contains <- lapply(cl, function(x) getClass(x, TRUE)@contains)
+    clnames <- c(clnames,
+                 unlist(lapply(contains, names), use.names=FALSE))
+    cltab <- table(factor(clnames, unique(clnames)))
+    clnames <- names(cltab)[cltab == length(x)]
+    if (length(clnames) > 0L) {
+      clnames[1]
+    } else {
+      NULL
+    }
   }
-  listClass[clExists[1L]]
 }
 
 coerceToList <- function(from, element.type = NULL, ...) {
-  if (is(from, listClassName(NULL, element.type)))
+  if (is(from, listClassName(NULL, element.type))) {
     return(from)
-  coerceToCompressedList(from, element.type, ...)
+  }
+  if (is.null(element.type)) {
+    if (is.list(from)) {
+      element.type <- listElementType(from)
+    } else {
+      element.type <- class(from)
+    }
+  }
+  as(from, selectListClassName(element.type))
 }
 
 setAs("ANY", "List", function(from) {
@@ -675,14 +832,6 @@ setMethod("unlist", "List",
     }
 )
 
-setMethod("unsplit", "List", function(value, f, drop = FALSE) {
-  value_flat <- unlist(value, use.names = FALSE)
-  if (length(value_flat) != length(f))
-    stop("Length of 'unlist(value)' must equal length of 'f'")
-  seqsplit(value_flat, f, drop = drop) <- value
-  value_flat
-})
-
 .stack.ind <- function(x, index.var = "name") {
   if (length(names(x)) > 0) {
     spaceLabels <- names(x)
@@ -713,22 +862,6 @@ setMethod("stack", "List",
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Evaluating.
 ###
-  
-setMethod("eval", c("expression", "List"),
-    function(expr, envir, enclos = parent.frame())
-          eval(expr, as.env(envir, enclos))
-)
-
-setMethod("eval", c("language", "List"),
-    function(expr, envir, enclos = parent.frame())
-          eval(expr, as.env(envir, enclos))
-)
-
-setMethod("with", "List",
-          function(data, expr, ...)
-          {
-            eval(substitute(expr), data, parent.frame())
-          })
 
 setMethod("within", "List",
           function(data, expr, ...)
@@ -736,7 +869,7 @@ setMethod("within", "List",
             ## cannot use active bindings here, as they break for replacement
             e <- list2env(as.list(data))
             ##e <- as.env(data)
-            eval(substitute(expr), e, parent.frame(2))
+            eval(substitute(expr), e, top_prenv(expr))
             l <- mget(ls(e), e)
             l <- l[!sapply(l, is.null)]
             nD <- length(del <- setdiff(names(data), (nl <- names(l))))
@@ -746,3 +879,9 @@ setMethod("within", "List",
               data[[nm]] <- NULL
             data
           })
+
+setMethod("do.call", c("ANY", "List"),
+          function (what, args, quote = FALSE, envir = parent.frame()) {
+            args <- as.list(args)
+            callGeneric()
+          })
