@@ -481,8 +481,8 @@ setMethod("[", "RangedData",
                 }
                 isplit <- split(i, igroup)
                 names(isplit) <- names(x)
-                ranges <- subsetListByList(ranges, isplit)
-                values <- subsetListByList(values, isplit)
+                ranges <- subset_List_by_List(ranges, isplit)
+                values <- subset_List_by_List(values, isplit)
                 if (drop) {
                   ok <- (elementLengths(ranges) > 0)
                   ranges <- ranges[ok]
@@ -633,7 +633,7 @@ setAs("RleViewsList", "RangedData", function(from) {
   from_ranges <- restrict(ranges(from), 1L, elementLengths(subject),
                           keep.all.ranges = TRUE)
 ### FIXME: do we want to insert NAs for out of bounds views?
-  score <- extractROWS(subject, from_ranges)
+  score <- subject[from_ranges]
   score_part <- seqapply(width(from_ranges), PartitioningByWidth)
   score_ranges <- ranges(score)
   ol <- findOverlaps(score_ranges, score_part)
