@@ -283,22 +283,6 @@ IRangesList <- function(..., universe = NULL, compress = TRUE)
 ### Subsetting.
 ###
 
-### Support subsetting by another RangesList
-rangesListSingleSquareBracket <- function(x, i, j, ..., drop)
-{
-  if (!missing(j) || length(list(...)) > 0)
-    stop("invalid subsetting")
-  if (missing(i))
-    return(x)
-  if (is(i, "RangesList"))
-    stop("'[' subsetting by RangesList is defunct.\n",
-         "Use 'subsetByOverlaps' instead.")
-  callNextMethod(x, i)
-}
-setMethod("[", "SimpleRangesList", rangesListSingleSquareBracket)
-setMethod("[", "CompressedIRangesList", rangesListSingleSquareBracket)
-setMethod("[", "SimpleIRangesList", rangesListSingleSquareBracket)
-
 setMethod("getListElement", "CompressedNormalIRangesList",
     function(x, i, exact=TRUE) newNormalIRangesFromIRanges(callNextMethod())
 )
@@ -431,15 +415,16 @@ as.data.frame.RangesList <- function(x, row.names=NULL, optional=FALSE, ...)
 }
 setMethod("as.data.frame", "RangesList", as.data.frame.RangesList)
 
-.as.list.CompressedNormalIRangesList <- function(x, use.names = TRUE)
-{
-    .CompressedList.list.subscript(X = x,
-                                   INDEX = seq_len(length(x)),
-                                   USE.NAMES = use.names,
-                                   FUN = newNormalIRangesFromIRanges,
-                                   COMPRESS = FALSE)
-}
 ### S3/S4 combo for as.list.CompressedNormalIRangesList
+.as.list.CompressedNormalIRangesList <- function(x, use.names=TRUE)
+{
+    if (!isTRUEorFALSE(use.names))
+        stop("'use.names' must be TRUE or FALSE")
+    ans <- lapply_CompressedList(x, newNormalIRangesFromIRanges)
+    if (use.names)
+        names(ans) <- names(x)
+    ans
+}
 as.list.CompressedNormalIRangesList <- function(x, ...)
     .as.list.CompressedNormalIRangesList(x, ...)
 setMethod("as.list", "CompressedNormalIRangesList",
