@@ -96,6 +96,9 @@ setMethod("Rle", signature = c(values = "vectorORfactor", lengths = "numeric"),
               Rle(values = values, lengths = as.integer(lengths),
                   check = check))
 
+setMethod("Rle", signature = c(values = "Rle", lengths = "missing"),
+          function(values, lengths, check = TRUE) values)
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Coercion
 ###
@@ -557,6 +560,10 @@ setMethod("is.na", "Rle",
               Rle(values = is.na(runValue(x)), lengths = runLength(x),
                   check = FALSE))
 
+setMethod("anyNA", "Rle",
+          function(x)
+              anyNA(runValue(x)))
+
 setMethod("is.unsorted", "Rle",
           function(x, na.rm = FALSE, strictly = FALSE)
           {
@@ -893,6 +900,49 @@ window.Rle <- function(x, start=NA, end=NA, width=NA,
 }
 setMethod("window", "Rle", window.Rle)
 
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### Set methods
+###
+### The return values of these do not have any duplicated values, so
+### it would obviously be more efficient to return plain vectors. That
+### might violate user expectations though.
+
+setMethod("union", c("Rle", "Rle"), function(x, y) {
+  Rle(union(runValue(x), runValue(y)))
+})
+
+setMethod("union", c("ANY", "Rle"), function(x, y) {
+  Rle(union(as.vector(x), runValue(y)))
+})
+
+setMethod("union", c("Rle", "ANY"), function(x, y) {
+  Rle(union(runValue(x), as.vector(y)))
+})
+
+setMethod("intersect", c("Rle", "Rle"), function(x, y) {
+  Rle(intersect(runValue(x), runValue(y)))
+})
+
+setMethod("intersect", c("ANY", "Rle"), function(x, y) {
+  Rle(intersect(as.vector(x), runValue(y)))
+})
+
+setMethod("intersect", c("Rle", "ANY"), function(x, y) {
+  Rle(intersect(runValue(x), as.vector(y)))
+})
+
+setMethod("setdiff", c("Rle", "Rle"), function(x, y) {
+  Rle(setdiff(runValue(x), runValue(y)))
+})
+
+setMethod("setdiff", c("ANY", "Rle"), function(x, y) {
+  Rle(setdiff(as.vector(x), runValue(y)))
+})
+
+setMethod("setdiff", c("Rle", "ANY"), function(x, y) {
+  Rle(setdiff(runValue(x), as.vector(y)))
+})
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Other logical data methods
 ###
@@ -1109,8 +1159,6 @@ setMethod("median", "Rle",
     callNextMethod(x=x, na.rm=FALSE)
 })
 
-### S3/S4 combo for quantile.Rle
-### FIXME: code duplication needed for S3 / S4 dispatch
 quantile.Rle <- 
     function(x, probs = seq(0, 1, 0.25), na.rm = FALSE, names = TRUE,
              type = 7, ...)
@@ -1122,17 +1170,6 @@ quantile.Rle <-
     on.exit(options("dropRle" = oldOption))
     NextMethod("quantile", na.rm=FALSE)
 }
-setMethod("quantile", "Rle",
-    function(x, probs = seq(0, 1, 0.25), na.rm = FALSE, names = TRUE,
-             type = 7, ...)
-{
-    if (na.rm)
-        x <- x[!is.na(x)]
-    oldOption <- getOption("dropRle")
-    options("dropRle" = TRUE)
-    on.exit(options("dropRle" = oldOption))
-    callNextMethod(x=x, probs=probs, na.rm=FALSE, names=names, type=type, ...)
-})
 
 setMethod("mad", "Rle",
           function(x, center = median(x), constant = 1.4826, na.rm = FALSE,
@@ -1420,6 +1457,15 @@ setReplaceMethod("levels", "Rle",
                      x
                  })
 
+droplevels.Rle <- function(x) {
+  if (!is.factor(runValue(x))) {
+    stop("levels can only be dropped when runValue(x) is a factor")
+  }
+  runValue(x) <- droplevels(runValue(x))
+  x
+}
+setMethod("droplevels", "Rle", droplevels.Rle)
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### The "show" method
 ###
