@@ -610,8 +610,8 @@ rbind.mcols <- function(x, ...)
     no_mcols <- sapply(l_mcols, is.null)
     if (all(no_mcols))
         return(NULL)
-    newDf <- function(nr) new("DataFrame", nrows = nr)
-    l_mcols[no_mcols] <- lapply(elementLengths(l[no_mcols]), newDf)
+    newDf <- function(xi) new("DataFrame", nrows = length(xi))
+    l_mcols[no_mcols] <- lapply(l[no_mcols], newDf)
     allCols <- unique(do.call(c, lapply(l_mcols, colnames)))
     fillCols <- function(df) {
       if (nrow(df))
