@@ -19,10 +19,7 @@ setMethod("shift", "Ranges",
             stop("'shift' must be a single number when shifting ",
                  "a NormalIRanges object")
         shift <- recycleIntegerArg(shift, "shift", length(x))
-        ## Note that we won't do anything with 'new_end', we just want to
-        ## fail in case of integer overflow.
-        errorIfWarning(new_start <- start(x) + shift)
-        errorIfWarning(new_end <- new_start + width(x) - 1L)
+        new_start <- start(x) + shift
         if (is(x, "IRanges")) {
             x@start <- new_start
         } else {
@@ -30,6 +27,7 @@ setMethod("shift", "Ranges",
         }
         if (!normargUseNames(use.names))
             names(x) <- NULL
+        validObject(x)
         x
     }
 )
@@ -189,7 +187,7 @@ setMethod("flank", "Ranges",
         width <- recycleIntegerArg(width, "width", length(x))
         if (!is.logical(start) || anyMissing(start))
             stop("'start' must be logical without NA's")
-        start <- recycleVector(start, length(x))
+        start <- recycleVector(unname(start), length(x))
         if (!isTRUEorFALSE(both))
             stop("'both' must be TRUE or FALSE")
         if (both) {
@@ -420,7 +418,7 @@ setMethod("resize", "IntervalForest",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### The "restrict" method.
+### restrict()
 ###
 
 setGeneric("restrict", signature="x",
@@ -590,8 +588,9 @@ setMethod("restrict", "IntervalForest",
                "IntervalForest")
           })
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### The "threebands" method.
+### threebands()
 ###
 
 setGeneric("threebands", signature="x",
@@ -651,7 +650,7 @@ setMethod("Ops", c("Ranges", "numeric"),
     }
 )
 
-setMethod("Ops", c("RangesList", "ANY"),
+setMethod("Ops", c("RangesList", "numeric"),
           function(e1, e2)
           {
             for (i in seq_len(length(e1)))
@@ -659,7 +658,7 @@ setMethod("Ops", c("RangesList", "ANY"),
             e1
           })
 
-setMethod("Ops", c("CompressedIRangesList", "ANY"),
+setMethod("Ops", c("CompressedIRangesList", "numeric"),
           function(e1, e2)
           {
             relist(callGeneric(unlist(e1, use.names = FALSE), e2), e1)
