@@ -344,6 +344,7 @@ const char *_get_classname(SEXP x);
 
 SEXP sapply_NROW(SEXP x);
 
+SEXP top_prenv(SEXP nm, SEXP env);
 
 /* int_utils.c */
 
@@ -511,6 +512,8 @@ SEXP findIntervalAndStartFromWidth(
 
 /* str_utils.c */
 
+SEXP unstrsplit_list(SEXP x, SEXP sep);
+
 SEXP safe_strexplode(SEXP s);
 
 SEXP strsplit_as_list_of_ints(SEXP x, SEXP sep);
@@ -571,6 +574,15 @@ SEXP vector_seqselect(
 );
 
 
+/* Ranges_class.c */
+
+SEXP valid_Ranges(
+	SEXP x_start,
+	SEXP x_end,
+	SEXP x_width
+);
+
+
 /* Ranges_comparison.c */
 
 int _overlap_code(
@@ -604,32 +616,32 @@ SEXP _get_IRanges_names(SEXP x);
 
 int _get_IRanges_length(SEXP x);
 
-cachedIRanges _cache_IRanges(SEXP x);
+IRanges_holder _hold_IRanges(SEXP x);
 
-int _get_cachedIRanges_length(const cachedIRanges *cached_x);
+int _get_length_from_IRanges_holder(const IRanges_holder *x_holder);
 
-int _get_cachedIRanges_elt_width(
-	const cachedIRanges *cached_x,
+int _get_width_elt_from_IRanges_holder(
+	const IRanges_holder *x_holder,
 	int i
 );
 
-int _get_cachedIRanges_elt_start(
-	const cachedIRanges *cached_x,
+int _get_start_elt_from_IRanges_holder(
+	const IRanges_holder *x_holder,
 	int i
 );
 
-int _get_cachedIRanges_elt_end(
-	const cachedIRanges *cached_x,
+int _get_end_elt_from_IRanges_holder(
+	const IRanges_holder *x_holder,
 	int i
 );
 
-SEXP _get_cachedIRanges_elt_name(
-	const cachedIRanges *cached_x,
+SEXP _get_names_elt_from_IRanges_holder(
+	const IRanges_holder *x_holder,
 	int i
 );
 
-cachedIRanges _sub_cachedIRanges(
-	const cachedIRanges *cached_x,
+IRanges_holder _get_linear_subset_from_IRanges_holder(
+	const IRanges_holder *x_holder,
 	int offset,
 	int length
 );
@@ -666,7 +678,7 @@ SEXP _alloc_IRanges(
 	int length
 );
 
-int _is_normal_cachedIRanges(const cachedIRanges *cached_ir);
+int _is_normal_IRanges_holder(const IRanges_holder *x_holder);
 
 SEXP IRanges_isNormal(SEXP x);
 
@@ -915,19 +927,19 @@ SEXP RleViews_viewWhichMaxs(
 
 /* CompressedIRangesList_class.c */
 
-cachedCompressedIRangesList _cache_CompressedIRangesList(SEXP x);
+CompressedIRangesList_holder _hold_CompressedIRangesList(SEXP x);
 
-int _get_cachedCompressedIRangesList_length(
-	const cachedCompressedIRangesList *cached_x
+int _get_length_from_CompressedIRangesList_holder(
+	const CompressedIRangesList_holder *x_holder
 );
 
-cachedIRanges _get_cachedCompressedIRangesList_elt(
-	const cachedCompressedIRangesList *cached_x,
+IRanges_holder _get_elt_from_CompressedIRangesList_holder(
+	const CompressedIRangesList_holder *x_holder,
 	int i
 );
 
-int _get_cachedCompressedIRangesList_eltLength(
-	const cachedCompressedIRangesList *cached_x,
+int _get_eltlens_from_CompressedIRangesList_holder(
+	const CompressedIRangesList_holder *x_holder,
 	int i
 );
 
@@ -984,7 +996,7 @@ SEXP Ranges_reduce(
 	SEXP x_width,
 	SEXP drop_empty_ranges,
 	SEXP min_gapwidth,
-	SEXP with_mapping,
+	SEXP with_revmap,
 	SEXP with_inframe_start
 );
 
@@ -992,7 +1004,7 @@ SEXP CompressedIRangesList_reduce(
 	SEXP x,
 	SEXP drop_empty_ranges,
 	SEXP min_gapwidth,
-	SEXP with_mapping
+	SEXP with_revmap
 );
 
 SEXP IRanges_gaps(
@@ -1014,45 +1026,6 @@ SEXP Ranges_disjointBins(
 );
 
 
-/* encodeOverlaps_methods.c */
-
-SEXP encode_overlaps1(
-	SEXP query_start,
-	SEXP query_width,
-	SEXP query_space,
-	SEXP query_break,
-	SEXP flip_query,
-	SEXP subject_start,
-	SEXP subject_width,
-	SEXP subject_space,
-	SEXP as_matrix,
-	SEXP as_raw
-);
-
-SEXP RangesList_encode_overlaps(
-	SEXP query_starts,
-	SEXP query_widths,
-	SEXP query_spaces,
-	SEXP query_breaks,
-	SEXP subject_starts,
-	SEXP subject_widths,
-	SEXP subject_spaces
-);
-
-SEXP Hits_encode_overlaps(
-	SEXP query_starts,
-	SEXP query_widths,
-	SEXP query_spaces,
-	SEXP query_breaks,
-	SEXP subject_starts,
-	SEXP subject_widths,
-	SEXP subject_spaces,
-	SEXP query_hits,
-	SEXP subject_hits,
-	SEXP flip_query
-);
-
-
 /* coverage_methods.c */
 
 SEXP IRanges_coverage(
